# iBiz GitLab-CI/CD模板

（技术引用链接）

`GitLab-CI/CD官方文档`[:arrow_upper_right:](https://docs.gitlab.com/ce/ci/yaml/README.html#artifacts)

`iBizSys模型api文档`[:arrow_upper_right:](https://modelapi.ibizlab.cn/)


## 介绍说明
在GitLab-CI/CD之前，iBizSys实现了基于Jenkins的CI/CD。

`GitLab CI`

GitLab CI是GitLab 提供的持续集成服务(从8.0版本之后，GitLab CI已经集成在GitLab中了)，只要在你的仓库根目录下创建一个.gitlab-ci.yml 文件， 并为该项目指派一个Runner，当有合并请求或者Push操作时，你写在.gitlab-ci.yml中的构建脚本就会开始执行。

`GitLab Runner`

GitLab Runner是配合GitLab CI进行构建任务的应用程序，GitLab CI负责yml文件中各种阶段流程的执行，而GitLab Runner就是具体的负责执行每个阶段的脚本执行。

`iBizSys`

iBizSys 是以 `Model Driven Development(MDD)`作为核心理论依据的开发框架，iBizSys从`2009年`诞生至今已经经历数个版本，构建出一套完整的系统构建模型，包含: `DevOps模型`、`文档模型`、`流程模型`、`测试模型`等。 iBizSys 提倡以`模型`作为主要内容的高级别抽象的开发方法，并结合市面上较为流行的技术框架形成技术模板，通过`模型`+`模板`的方式快速构建出一套生产级并且具备`复杂性`和`变更能力`的系统。



## 快速开始

## GitLab-CI/CD实现  

我们将通过搭建一个简单的Demo来阐述 iBizSys 实现 GitLab-CI/CD 的过程，在此之前，我们假设您已经：

- **熟悉GitLab-CI/CD**

- **熟悉FreeMaker**
	


## 建立模板

由于MyBatis-Plus 示例中使用Spring Boot作为工程项目，所以本示例中也将使用Spring Boot作为工程项目。

搭建 SpringBoot 项目环境可通过以下两种方式进行：

>在iBizSys模板仓库中fork SpringBoot模板，基于SpringBoot的基础上增加 Mybatis-Plus 的模板配置 `推荐使用`

>自行搭建 SpringBoot 模板环境，并引入 Mybatis-Plus 相关的模板配置

本示例采用fork的方式获取SpringBoot的模板 [查看更多模板仓库的小知识](http://bbs.ibizlab.cn/)

`编写模板方式`

>在线编辑：`iBizSys` 提供模板在线编辑工具，供大家更高效、便捷的编写模板 [查看iBizSys在线编辑工具](http://bbs.ibizlab.cn/)

>离线编辑：由于`iBizSys`的模板是放在`git`仓库中进行托管，用户可在离线环境下编写模板，编辑完成后再将模板上传至`git`中即可。

### 依赖模板：

在 `pom.xml.ftl` 中引入 Mybatis-Plus 所需的相关依赖

```java

<dependency>
    <groupId>com.baomidou</groupId>
    <artifactId>mybatis-plus-boot-starter</artifactId>
    <version>3.0.6</version>
</dependency>
<dependency>
    <groupId>com.h2database</groupId>
    <artifactId>h2</artifactId>
</dependency>

```

### 配置模板：

在 `application.yml.ftl`中添加 H2 数据库的相关配置

```java

spring:
  datasource:
    username: test
    password: 'test'
    url: jdbc:h2:mem:demo;
    driver-class-name: org.h2.Driver
    schema: classpath:db/schema-h2.sql
    data: classpath:db/data-h2.sql
    
```


### 启动类模板：

在 启动类` %PUBPRJ%Main.java.ftl `中,添加 `@MapperScan` 注解，扫描 Mapper 文件夹。模板中含 ` % % 、${ } `均为动态参数 [查看模板参数](http://bbs.ibizlab.cn/)

```java

@SpringBootApplication
@MapperScan("${pub.getPKGCodeName()}.*.mapper")
public class ${pub.getCodeName()?lower_case}Main{

	public static void main(String[] args) {
		SpringApplication.run(${pub.getCodeName()?lower_case}Main.class, args);
	}
}


```

### 实体类模板：

实体类模板将循环输出该实体的所有属性，配置 `@TableName`  `@TableId` 注解，用于指定该实体所映射的数据库表及主键属性，以下为调整后的实体类相关的模板代码：

```java

@TableName(value = "${item.getTableName()}")
@Data
public class ${item.codeName}{

<#list item.getPSDEFields() as defield>
<#if defield.isKeyDEField()>
    @TableId(value= "${defield.getName()?lower_case}",type=IdType.UUID)//指定主键生成策略
</#if>
    private ${srfjavatype(defield.stdDataType)} ${defield.codeName?lower_case};
</#list>

}

```

### Mapper类模板：

创建一个Mapper类的模板文件【%DE%Mapper.java.ftl】

```java

public interface ${item.getCodeName()}Mapper extends BaseMapper<${item.getCodeName()}>{


}

```

## 预览成果物

iBizSys提供了即时预览，可以让您在编写模板的同时预览到最终成果物 [查看更多模板编写小技巧](http://bbs.ibizlab.cn/)

>图中左侧为模板的成果物

>图中间为模板代码

>图右侧为动态运行时，可通过动态运行时查看模板所需参数

![produce](img/quickstart/result_preview_de.png)

## 发布模板

您在iBizSys模板工具编写模板后，模板处于暂存状态，若您需要在项目中应用该模板，需要发布该模板。

![produce](img/quickstart/publishFtl.png)

## 发布模型

发布模型的过程即为从模板仓库中获取最新模板合成模型，生产代码的过程。

![produce](img/quickstart/publishSystem.png)

## 查看最终成果物

### 依赖

![produce](img/quickstart/result_pom.png)

### 配置

![produce](img/quickstart/result_yml.png)

### 启动类

![produce](img/quickstart/result_appMain.png)

### 实体类

![produce](img/quickstart/result_de.png)

### Mapper类

![produce](img/quickstart/result_mapper.png)


### 开始使用



添加`DBET`实体建表的schema脚本:

```sql

create table IF NOT EXISTS  T_DBET
(
  dbetid      VARCHAR2(100) not null,
  createman  VARCHAR2(60),
  updateman  VARCHAR2(60),
  createdate DATE,
  dbetname    VARCHAR2(200),
  updatedate DATE
);

```

添加`DBET`实体测试数据的schema脚本


```sql

INSERT INTO T_DBET (dbetid, dbetname) VALUES
(1, '这是第一条实体数据'),
(2, '这是第二条实体数据'),
(3, '这是第三条实体数据'),
(4, '这是第四条实体数据'),
(5, '这是第五条实体数据');

```

添加测试类，进行功能测试，测试类相关代码如下：

```java

@RunWith(SpringRunner.class)
@SpringBootTest(classes = demoMain.class)
public class QuickStartTest {
    @Autowired
    private DBETMapper dbetMapper;

    @Test
    public void testMain() {
        System.out.println(("----- selectAll method test ------"));
        List<DBET> userList =  dbetMapper.selectList(null);
        Assert.assertEquals(5, userList.size());
        userList.forEach(System.out::println);
    }
}

```

控制台输出：

```
----- selectAll method test ------
DBET(dbetname=这是第一条实体数据, createdate=null, createman=null, dbetid=1, updateman=null, updatedate=null)
DBET(dbetname=这是第二条实体数据, createdate=null, createman=null, dbetid=2, updateman=null, updatedate=null)
DBET(dbetname=这是第三条实体数据, createdate=null, createman=null, dbetid=3, updateman=null, updatedate=null)
DBET(dbetname=这是第四条实体数据, createdate=null, createman=null, dbetid=4, updateman=null, updatedate=null)
DBET(dbetname=这是第五条实体数据, createdate=null, createman=null, dbetid=5, updateman=null, updatedate=null)
 
```

## 小结

通过以上几个简单的步骤，您已经成功搭建一个（SpringBoot+MyBatis-Plus）的项目，实现了对数据表的CURD功能。并且您也成功了搭建了一套属于您个人的技术模板，您可以在以后的任何项目当中，使用您搭建的这套技术模板。

iBizSys可以帮助您快速搭建并使用您所搭建的技术模板来生产您的项目。